"""Module for interacting with a JVC Projector."""

from __future__ import annotations

import logging

from . import command, const
from .command import JvcCommand
from .connection import resolve
from .device import JvcDevice
from .error import JvcProjectorConnectError, JvcProjectorError

_LOGGER = logging.getLogger(__name__)

DEFAULT_PORT = 20554
DEFAULT_TIMEOUT = 15.0


class JvcProjector:
    """Class for interacting with a JVC Projector."""

    def __init__(
        self,
        host: str,
        *,
        port: int = DEFAULT_PORT,
        timeout: float = DEFAULT_TIMEOUT,
        password: str | None = None,
    ) -> None:
        """Initialize class."""
        self._host = host
        self._port = port
        self._timeout = timeout
        self._password = password

        self._device: JvcDevice | None = None
        self._ip: str = ""
        self._model: str = ""
        self._mac: str = ""
        self._version: str = ""

    @property
    def ip(self) -> str:
        """Returns ip."""
        if not self._ip:
            raise JvcProjectorError("ip not initialized")
        return self._ip

    @property
    def host(self) -> str:
        """Returns host."""
        return self._host

    @property
    def port(self) -> int:
        """Returns port."""
        return self._port

    @property
    def model(self) -> str:
        """Returns model name."""
        if not self._mac:
            raise JvcProjectorError("model not initialized")
        return self._model

    @property
    def mac(self) -> str:
        """Returns mac address."""
        if not self._mac:
            raise JvcProjectorError("mac address not initialized")
        return self._mac

    @property
    def version(self) -> str:
        """Get device software version."""
        if not self._version:
            raise JvcProjectorError("version address not initialized")
        return self._version

    async def connect(self, get_info: bool = False) -> None:
        """Connect to device."""
        if self._device:
            return

        if not self._ip:
            self._ip = await resolve(self._host)

        self._device = JvcDevice(self._ip, self._port, self._timeout, self._password)

        if not await self.test():
            raise JvcProjectorConnectError("Failed to verify connection")

        if get_info:
            await self.get_info()

    async def disconnect(self) -> None:
        """Disconnect from device."""
        if self._device:
            await self._device.disconnect()
            self._device = None

    async def get_info(self) -> dict[str, str]:
        """Get device info."""
        assert self._device
        model = JvcCommand(command.MODEL, True)
        mac = JvcCommand(command.MAC, True)
        await self._send([model, mac])

        if mac.response is None:
            raise JvcProjectorError("Mac address not available")

        if model.response is None:
            model.response = "(unknown)"

        self._model = model.response
        self._mac = mac.response

        return {"model": self._model, "mac": self._mac}

    async def get_state(self) -> dict[str, str | None]:
        """Get device state."""
        assert self._device
        pwr = JvcCommand(command.POWER, True)
        inp = JvcCommand(command.INPUT, True)
        src = JvcCommand(command.SOURCE, True)
        res = await self._send([pwr, inp, src])
        return {
            "power": res[0] or None,
            "input": res[1] or const.NOSIGNAL,
            "source": res[2] or const.NOSIGNAL,
        }

    async def get_version(self) -> str | None:
        """Get device software version."""
        return await self.ref(command.VERSION)

    async def get_power(self) -> str | None:
        """Get power state."""
        return await self.ref(command.POWER)

    async def get_input(self) -> str | None:
        """Get current input."""
        return await self.ref(command.INPUT)

    async def get_signal(self) -> str | None:
        """Get if has signal."""
        return await self.ref(command.SOURCE)

    async def test(self) -> bool:
        """Run test command."""
        cmd = JvcCommand(f"{command.TEST}")
        await self._send([cmd])
        return cmd.ack

    async def power_on(self) -> None:
        """Run power on command."""
        await self.op(f"{command.POWER}1")

    async def power_off(self) -> None:
        """Run power off command."""
        await self.op(f"{command.POWER}0")

    async def remote(self, code: str) -> None:
        """Run remote code command."""
        await self.op(f"{command.REMOTE}{code}")

    async def op(self, code: str) -> None:
        """Send operation code."""
        await self._send([JvcCommand(code, False)])

    async def ref(self, code: str) -> str | None:
        """Send reference code."""
        return (await self._send([JvcCommand(code, True)]))[0]

    async def _send(self, cmds: list[JvcCommand]) -> list[str | None]:
        """Send command to device."""
        if self._device is None:
            raise JvcProjectorError("Must call connect before sending commands")

        await self._device.send(cmds)

        return [cmd.response for cmd in cmds]



"""Module for representing a JVC Projector command."""

from __future__ import annotations

from collections.abc import Callable
import logging
import re
from typing import Final

from . import const

_LOGGER = logging.getLogger(__name__)

PJOK: Final = b"PJ_OK"
PJNG: Final = b"PJ_NG"
PJREQ: Final = b"PJREQ"
PJACK: Final = b"PJACK"
PJNAK: Final = b"PJNAK"

UNIT_ID: Final = b"\x89\x01"
HEAD_OP: Final = b"!" + UNIT_ID
HEAD_REF: Final = b"?" + UNIT_ID
HEAD_RES: Final = b"@" + UNIT_ID
HEAD_ACK: Final = b"\x06" + UNIT_ID
HEAD_LEN: Final = 1 + len(UNIT_ID)
END: Final = b"\n"

TEST: Final = "\0\0"
VERSION: Final = "IFSV"
MODEL: Final = "MD"
MAC: Final = "LSMA"
SOURCE: Final = "SC"
POWER: Final = "PW"
INPUT: Final = "IP"
REMOTE: Final = "RC"

AUTH_SALT: Final = "JVCKWPJ"


class JvcCommand:
    """Class for representing a JVC Projector command."""

    def __init__(self, code: str, is_ref=False):
        """Initialize class."""
        self.code = code
        self.is_ref = is_ref
        self.ack = False
        self._response: str | None = None

    @property
    def response(self) -> str | None:
        """Return command response."""
        if not self.is_ref or self._response is None:
            return None

        res = self.code + self._response
        val: str = res[len(self.code) :]

        for pat, fmt in self.formatters.items():
            m = re.search(r"^" + pat + r"$", res)
            if m:
                if isinstance(fmt, list):
                    try:
                        return fmt[int(m[1], 16)]
                    except ValueError:
                        msg = "response '%s' not int for cmd '%s'"
                        _LOGGER.warning(msg, val, self.code)
                    except KeyError:
                        msg = "response '%s' not mapped for cmd '%s'"
                        _LOGGER.warning(msg, val, self.code)
                elif isinstance(fmt, dict):
                    try:
                        return fmt[m[1]]
                    except KeyError:
                        msg = "response '%s' not mapped for cmd '%s'"
                        _LOGGER.warning(msg, val, self.code)
                elif callable(fmt):
                    try:
                        return fmt(m)
                    except Exception as e:  # noqa: BLE001
                        msg = "response format failed with %s for '%s (%s)'"
                        _LOGGER.warning(msg, e, self.code, val)
                break

        return val

    @response.setter
    def response(self, data: str) -> None:
        """Set command response."""
        self._response = data

    @property
    def is_power(self) -> bool:
        """Return if command is a power command."""
        return self.code.startswith("PW")

    formatters: dict[str, list | dict | Callable] = {
        # Power
        "PW(.)": [const.STANDBY, const.ON, const.COOLING, const.WARMING, const.ERROR],
        # Input
        "(?:IP|IFIN)(.)": {
            "0": "svideo",
            "1": "video",
            "2": "component",
            "3": "pc",
            "6": const.HDMI1,
            "7": const.HDMI2,
        },
        # Source
        "SC(.)": [const.NOSIGNAL, const.SIGNAL],
        # Model
        "MD(.+)": lambda r: re.sub(r"-+", "-", r[1].replace(" ", "-")),
        # Picture Mode
        "PMPM(..)": {
            "00": "film",
            "01": "cinema",
            "02": "animation",
            "03": "natural",
            "04": "hdr10",
            "06": "thx",
            "0B": "frameadapt_hdr",
            "0C": "user1",
            "0D": "user2",
            "0E": "user3",
            "0F": "user4",
            "10": "user5",
            "11": "user6",
            "14": "hlg",
            "15": "hdr10+",
            "16": "pana_pq",
            "17": "filmmaker",
            "18": "frameadapt_hdr2",
            "19": "frameadapt_hdr3",
        },
        # Picture Mode - Intelligent Lens Aperture
        "PMDI(.)": ["off", "auto1", "auto2"],
        # Picture Mode - Color Profile
        "PMPR(..)": {
            "00": "off",
            "01": "film1",
            "02": "film2",
            "03": "bt709",
            "04": "cinema",
            "05": "cinema2",
            "06": "anime",
            "07": "anime2",
            "08": "video",
            "09": "vivid",
            "0A": "hdr",
            "0B": "bt2020(wide)",
            "0C": "3d",
            "0D": "thx",
            "0E": "custom1",
            "0F": "custom2",
            "10": "custom3",
            "11": "custom4",
            "12": "custom5",
            "21": "dci",
            "22": "custom6",
            "24": "bt2020(normal)",
            "25": "off(wide)",
            "26": "auto",
        },
        # Picture Mode - Color Temp
        "PMCL(..)": {
            "00": "5500k",
            "02": "6500k",
            "04": "7500k",
            "08": "9300k",
            "09": "high",
            "0A": "custom1",
            "0B": "custom2",
            "0C": "hdr10",
            "0D": "xenon1",
            "0E": "xenon2",
            "14": "hlg",
        },
        # Picture Mode - Color Correction
        "PMCC(..)": {
            "00": "5500k",
            "02": "6500k",
            "04": "7500k",
            "08": "9300k",
            "09": "high",
            "0D": "xenon1",
            "0E": "xenon2",
        },
        # Picture Mode - Gamma Table
        "PMGT(..)": {
            "00": "2.2",
            "01": "cinema1",
            "02": "cinema2",
            "04": "custom1",
            "05": "custom2",
            "06": "custom3",
            "07": "hdr_hlg",
            "08": "2.4",
            "09": "2.6",
            "0A": "film1",
            "0B": "film2",
            "0C": "hdr_pq",
            "0D": "pana_pq",
            "10": "thx",
            "15": "hdr_auto",
        },
        # Picture Mode - Color Management, Low Latency, 8K E-Shift
        "PM(?:CB|LL|US)(.)": ["off", "on"],
        # Picture Mode - Clear Motion Drive
        "PMCM(.)": ["off", None, None, "low", "high", "inverse_telecine"],
        # Picture Mode - Motion Enhance
        "PMME(.)": ["off", "low", "high"],
        # Picture Mode - Lamp Power
        "PMLP(.)": ["normal", "high"],
        # Picture Mode - Graphics Mode
        "PMGM(.)": ["standard", "high-res"],
        # Input Signal - HDMI Input Level
        "ISIL(.)": ["standard", "enhanced", "super_white", "auto"],
        # Input Signal - HDMI Color Space
        "ISHS(.)": ["auto", "ycbcr(4:4:4)", "ycbcr(4:2:2)", "rgb"],
        # Input Signal - HDMI 2D/3D
        "IS3D(.)": ["2d", "auto", None, "side_by_side", "top_bottom"],
        # Input Signal - Aspect
        "ISAS(.)": [None, None, "zoom", "auto", "native"],
        # Input Signal - Mask
        "ISMA(.)": [None, "on", "off"],
        # Installation - Lens Control
        "IN(?:FN|FF|ZT|ZW|SL|SR|SU|SD)(.)": ["stop", "start"],
        # Installation - Lens Image Pattern, Lens Lock, Screen Adjust
        "IN(?:IP|LL|SC|HA)(.)": ["off", "on"],
        # Installation - Style
        "INIS(.)": ["front", "front_ceiling", "rear", "rear_ceiling"],
        # Installation - Anamorphic
        "INVS(.)": ["off", "a", "b", "c", "d"],
        # Display - Back Color
        "DSBC(.)": ["blue", "black"],
        # Display - Menu Positions
        "DSMP(.)": [
            "left-top",
            "right-top",
            "center",
            "left-bottom",
            "right-bottom",
            "left",
            "right",
        ],
        # Installation - Source Display, Logo Data
        "DS(?:SD|LO)(.)": ["off", "on"],
        # Function - Trigger
        "FUTR(.)": [
            "off",
            "power",
            "anamo",
            "ins1",
            "ins2",
            "ins3",
            "ins4",
            "ins5",
            "ins6",
            "ins7",
            "ins8",
            "ins9",
            "ins10",
        ],
        # Function - Off Timer
        "FUOT(.)": ["off", "1hour", "2hour", "3hour", "4hour"],
        # Function - Eco Mode, Control4
        "FU(?:EM|CF)(.)": ["off", "on"],
        # Function - Input
        "IFIN(.)": ["hdmi1", "hdmi2"],
        # Function - Source
        "IFIS(..)": {
            "02": "480p",
            "03": "576p",
            "04": "720p50",
            "05": "720p60",
            "06": "1080i50",
            "07": "1080i60",
            "08": "1080p24",
            "09": "1080p50",
            "0A": "1080p60",
            "0B": "nosignal",
            "0C": "720p3d",
            "0D": "1080i3d",
            "0E": "1080p3d",
            "0F": "outofrange",
            "10": "4k(4096)60",
            "11": "4k(4096)50",
            "12": "4k(4096)30",
            "13": "4k(4096)25",
            "14": "4k(4096)24",
            "15": "4k(3840)60",
            "16": "4k(3840)50",
            "17": "4k(3840)30",
            "18": "4k(3840)25",
            "19": "4k(3840)24",
            "1C": "1080p25",
            "1D": "1080p30",
            "1E": "2048x1080p24",
            "1F": "2048x1080p25",
            "20": "2048x1080p30",
            "21": "2048x1080p50",
            "22": "2048x1080p60",
            "23": "3840x2160p120",
            "24": "4096x2160p120",
            "25": "vga(640x480)",
            "26": "svga(800x600)",
            "27": "xga(1024x768)",
            "28": "sxga(1280x1024)",
            "29": "wxga(1280x768)",
            "2A": "wxga+(1440x900)",
            "2B": "wsxga+(1680x1050)",
            "2C": "wuxga(1920x1200)",
            "2D": "wxga(1280x800)",
            "2E": "fwxga(1366x768)",
            "2F": "wxga++(1600x900)",
            "30": "uxga(1600x1200)",
            "31": "qxga",
            "32": "wqxga",
        },
        # Function - Deep Color
        "IFDC(.)": ["8bit", "10bit", "12bit"],
        # Function - Color Space
        "IFXV(.)": ["rgb", "yuv"],
        # Function - Colorimetry
        "IFCM(.)": [
            "nodata",
            "bt601",
            "bt709",
            "xvycc601",
            "xvycc709",
            "sycc601",
            "adobe_ycc601",
            "adobe_rgb",
            "bt2020(constant_luminance)",
            "bt2020(non-constant_luminance)",
            "srgb",
        ],
        # Function - HDR
        "IFHR(.)": {
            "0": "sdr",
            "1": "hdr",
            "2": "smpte_st_2084",
            "3": "hybrid_log",
            "F": "none",
        },
        # Picture Mode - HDR Level
        "PMHL(.)": ["auto", "-2", "-1", "0", "1", "2"],
        # Picture Mode - HDR Processing
        "PMHP(.)": ["static", "frame", "scene"],
        # Picture Mode - HDR Processing
        "PMCT(.)": ["auto", "sdr", None, "hdr10", "hlg"],
        # Picture Mode - Theater Optimizer
        "PMNM(.)": ["off", "on"],
        # Picture Mode - Theater Optimizer Level
        "PMNL(.)": ["reserved", "low", "medium", "high"],
        # Picture Mode - Theater Optimizer Processing
        "PMNP(.)": ["-", "start"],
        # Lan Setup
        "LSDS(.)": [const.OFF, const.ON],
        "LSMA(.+)": lambda r: re.sub(r"-+", "-", r[1].replace(" ", "-")),
        "LSIP(..)(..)(..)(..)": lambda r: f"{int(r[1], 16)}.{int(r[2], 16)}.{int(r[3], 16)}.{int(r[4], 16)}",
    }



"""Tests for projector module."""

from unittest.mock import AsyncMock

import pytest

from jvcprojector import command, const
from jvcprojector.error import JvcProjectorError
from jvcprojector.projector import JvcProjector

from . import HOST, IP, MAC, MODEL, PORT


@pytest.mark.asyncio
async def test_init(dev: AsyncMock):
    """Test init succeeds."""
    p = JvcProjector(IP, port=PORT)
    assert p.host == IP
    assert p.port == PORT
    with pytest.raises(JvcProjectorError):
        assert p.ip
    with pytest.raises(JvcProjectorError):
        assert p.model
    with pytest.raises(JvcProjectorError):
        assert p.mac


@pytest.mark.asyncio
async def test_connect(dev: AsyncMock):
    """Test connect succeeds."""
    p = JvcProjector(IP, port=PORT)
    await p.connect()
    assert p.ip == IP
    await p.disconnect()
    assert dev.disconnect.call_count == 1


@pytest.mark.asyncio
async def test_connect_host(dev: AsyncMock):
    """Test connect succeeds."""
    p = JvcProjector(HOST, port=PORT)
    await p.connect()
    assert p.ip == IP
    await p.disconnect()
    assert dev.disconnect.call_count == 1


@pytest.mark.asyncio
@pytest.mark.parametrize("dev", [{command.MODEL: None}], indirect=True)
async def test_unknown_model(dev: AsyncMock):
    """Test projector with unknown model succeeds."""
    p = JvcProjector(IP)
    await p.connect()
    await p.get_info()
    assert p.mac == MAC
    assert p.model == "(unknown)"


@pytest.mark.asyncio
@pytest.mark.parametrize("dev", [{command.MAC: None}], indirect=True)
async def test_unknown_mac(dev: AsyncMock):
    """Test projector with unknown mac uses model succeeds."""
    p = JvcProjector(IP)
    await p.connect()
    with pytest.raises(JvcProjectorError):
        await p.get_info()


@pytest.mark.asyncio
async def test_get_info(dev: AsyncMock):
    """Test get_info succeeds."""
    p = JvcProjector(IP)
    await p.connect()
    assert await p.get_info() == {"model": MODEL, "mac": MAC}


@pytest.mark.asyncio
async def test_get_state(dev: AsyncMock):
    """Test get_state succeeds."""
    p = JvcProjector(IP)
    await p.connect()
    assert await p.get_state() == {
        "power": const.ON,
        "input": const.HDMI1,
        "source": const.SIGNAL,
    }



"""Tests for device module."""

from hashlib import sha256
from unittest.mock import AsyncMock, call

import pytest

from jvcprojector import command, const
from jvcprojector.command import JvcCommand
from jvcprojector.device import (
    AUTH_SALT,
    HEAD_ACK,
    HEAD_OP,
    HEAD_REF,
    HEAD_RES,
    PJACK,
    PJNAK,
    PJNG,
    PJOK,
    PJREQ,
    JvcDevice,
)
from jvcprojector.error import JvcProjectorCommandError

from . import IP, PORT, TIMEOUT, cc


@pytest.mark.asyncio
async def test_send_op(conn: AsyncMock):
    """Test send operation command succeeds."""
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(f"{command.POWER}1")
    await dev.send([cmd])
    await dev.disconnect()
    assert cmd.ack
    assert cmd.response is None
    conn.connect.assert_called_once()
    conn.write.assert_has_calls([call(PJREQ), call(cc(HEAD_OP, f"{command.POWER}1"))])


@pytest.mark.asyncio
async def test_send_ref(conn: AsyncMock):
    """Test send reference command succeeds."""
    conn.readline.side_effect = [
        cc(HEAD_ACK, command.POWER),
        cc(HEAD_RES, command.POWER + "1"),
    ]
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(command.POWER, True)
    await dev.send([cmd])
    await dev.disconnect()
    assert cmd.ack
    assert cmd.response == const.ON
    conn.connect.assert_called_once()
    conn.write.assert_has_calls([call(PJREQ), call(cc(HEAD_REF, command.POWER))])


@pytest.mark.asyncio
async def test_send_with_password8(conn: AsyncMock):
    """Test send with 8 character password succeeds."""
    dev = JvcDevice(IP, PORT, TIMEOUT, "passwd78")
    cmd = JvcCommand(f"{command.POWER}1")
    await dev.send([cmd])
    await dev.disconnect()
    conn.write.assert_has_calls(
        [call(PJREQ + b"_passwd78\x00\x00"), call(cc(HEAD_OP, f"{command.POWER}1"))]
    )


@pytest.mark.asyncio
async def test_send_with_password10(conn: AsyncMock):
    """Test send with 10 character password succeeds."""
    dev = JvcDevice(IP, PORT, TIMEOUT, "passwd7890")
    cmd = JvcCommand(f"{command.POWER}1")
    await dev.send([cmd])
    await dev.disconnect()
    conn.write.assert_has_calls(
        [call(PJREQ + b"_passwd7890"), call(cc(HEAD_OP, f"{command.POWER}1"))]
    )


@pytest.mark.asyncio
async def test_send_with_password_sha256(conn: AsyncMock):
    """Test send with a projector requiring sha256 hashing."""
    conn.read.side_effect = [PJOK, PJNAK, PJACK]
    dev = JvcDevice(IP, PORT, TIMEOUT, "passwd78901")
    cmd = JvcCommand(f"{command.POWER}1")
    await dev.send([cmd])
    await dev.disconnect()
    auth = sha256(f"passwd78901{AUTH_SALT}".encode()).hexdigest().encode()
    conn.write.assert_has_calls(
        [call(PJREQ + b"_" + auth), call(cc(HEAD_OP, f"{command.POWER}1"))]
    )


@pytest.mark.asyncio
@pytest.mark.parametrize("conn", [{"raise_on_connect": 1}], indirect=True)
async def test_connection_refused_retry(conn: AsyncMock):
    """Test connection refused results in retry."""
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(f"{command.POWER}1")
    await dev.send([cmd])
    await dev.disconnect()
    assert cmd.ack
    assert conn.connect.call_count == 2
    conn.write.assert_has_calls([call(PJREQ), call(cc(HEAD_OP, f"{command.POWER}1"))])


@pytest.mark.asyncio
async def test_connection_busy_retry(conn: AsyncMock):
    """Test handshake busy results in retry."""
    conn.read.side_effect = [PJNG, PJOK, PJACK]
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(f"{command.POWER}1")
    await dev.send([cmd])
    await dev.disconnect()
    assert conn.connect.call_count == 2
    conn.write.assert_has_calls([call(PJREQ), call(cc(HEAD_OP, f"{command.POWER}1"))])


@pytest.mark.asyncio
async def test_connection_bad_handshake_error(conn: AsyncMock):
    """Test bad handshake results in error."""
    conn.read.side_effect = [b"BAD"]
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(f"{command.POWER}1")
    with pytest.raises(JvcProjectorCommandError):
        await dev.send([cmd])
    conn.connect.assert_called_once()
    conn.disconnect.assert_called_once()
    assert not cmd.ack


@pytest.mark.asyncio
async def test_send_op_bad_ack_error(conn: AsyncMock):
    """Test send operation with bad ack results in error."""
    conn.readline.side_effect = [cc(HEAD_ACK, "ZZ")]
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(f"{command.POWER}1")
    with pytest.raises(JvcProjectorCommandError):
        await dev.send([cmd])
    conn.connect.assert_called_once()
    conn.disconnect.assert_called_once()
    assert not cmd.ack


@pytest.mark.asyncio
async def test_send_ref_bad_ack_error(conn: AsyncMock):
    """Test send reference with bad ack results in error."""
    conn.readline.side_effect = [cc(HEAD_ACK, command.POWER), cc(HEAD_RES, "ZZ1")]
    dev = JvcDevice(IP, PORT, TIMEOUT)
    cmd = JvcCommand(command.POWER, True)
    with pytest.raises(JvcProjectorCommandError):
        await dev.send([cmd])
    conn.connect.assert_called_once()
    conn.disconnect.assert_called_once()
    assert not cmd.ack



"""Device constants for a JVC Projector."""

from typing import Final

POWER: Final = "power"
OFF: Final = "off"
STANDBY: Final = "standby"
ON: Final = "on"
WARMING: Final = "warming"
COOLING: Final = "cooling"
ERROR: Final = "error"

INPUT: Final = "input"
HDMI1 = "hdmi1"
HDMI2 = "hdmi2"

SOURCE: Final = "source"
NOSIGNAL: Final = "nosignal"
SIGNAL: Final = "signal"

REMOTE_MENU: Final = "732E"
REMOTE_UP: Final = "7301"
REMOTE_DOWN: Final = "7302"
REMOTE_LEFT: Final = "7336"
REMOTE_RIGHT: Final = "7334"
REMOTE_OK: Final = "732F"
REMOTE_BACK: Final = "7303"
REMOTE_MPC: Final = "73F0"
REMOTE_HIDE: Final = "731D"
REMOTE_INFO: Final = "7374"
REMOTE_INPUT: Final = "7308"
REMOTE_ADVANCED_MENU: Final = "7373"
REMOTE_PICTURE_MODE: Final = "73F4"
REMOTE_COLOR_PROFILE: Final = "7388"
REMOTE_LENS_CONTROL: Final = "7330"
REMOTE_SETTING_MEMORY: Final = "73D4"
REMOTE_GAMMA_SETTINGS: Final = "73F5"
REMOTE_CMD: Final = "738A"
REMOTE_MODE_1: Final = "73D8"
REMOTE_MODE_2: Final = "73D9"
REMOTE_MODE_3: Final = "73DA"
REMOTE_MODE_4: Final = "73E5"
REMOTE_MODE_5: Final = "73E6"
REMOTE_MODE_6: Final = "73E7"
REMOTE_MODE_7: Final = "73E8"
REMOTE_MODE_8: Final = "73E9"
REMOTE_MODE_9: Final = "73EA"
REMOTE_MODE_10: Final = "73EB"
REMOTE_HDMI_1: Final = "7370"
REMOTE_HDMI_2: Final = "7371"
REMOTE_LENS_AP: Final = "7320"
REMOTE_ANAMO: Final = "73C5"
REMOTE_GAMMA: Final = "7375"
REMOTE_COLOR_TEMP: Final = "7376"
REMOTE_3D_FORMAT: Final = "73D6"
REMOTE_PIC_ADJ: Final = "7372"
REMOTE_NATURAL: Final = "736A"
REMOTE_CINEMA: Final = "7368"



"""A python library for controlling a JVC Projector over a network connection."""

# ruff: noqa: F401

from .error import (
    JvcProjectorAuthError,
    JvcProjectorCommandError,
    JvcProjectorConnectError,
    JvcProjectorError,
)
from .projector import JvcProjector

__version__ = "1.1.3"
